<?php
/* $Id: ganglia.php,v 1.6 2002/12/09 20:15:39 sacerdoti Exp $ */
#
# Parses full ganglia XML tree, and provides some useful functions.
#
# There are two modes here: "graphing" and "general". We start in
# graphing mode, where we attempt to generate graphs for all hosts
# that we encounter. If we find a child grid (one of our gmetad sources
# is a grid instead of a cluster) we switch
# to "general" mode. In general mode, we only display the summary graph
# of the grid, and do not attempt to track any clusters or hosts beneath
# it.
#
# The idea is to reduce processing when we know a gmetad below us has
# complete graphs of a cluster. We refer to them instead of duplicating
# their work locally. Designed by Sacerdoti (fds@sdsc.edu).
#

$error="";

# Timing the XML download and parsing may give some insight as to when this
# server is becoming overloaded. Necessary because we do not trim the XML
# tree, even in potentially large hierarchies. Gives time in seconds.
$parsetime;


#
# This is the global variables which hold all the information returned by
# gmetad.  I _wish_ this didnt have to be global but I cant find how
# how to set a user-define variable to pass to
# the element handlers in the PHP documentation.. I guess well have to wait
# to build a Cluster of Cluster of Clusters.*
#
# *(Sacerdoti) Here it is...
#

# 2key = "Grid Name" / "NAME | AUTHORITY | cpu_num | load_one | ... "
# All metrics are summarized over potentially many clusters. Only includes immediate
# child grids and clusters.
$grids = array();

# Contains the name, authority for all grids we can see.
$allgrids = array();

# 1Key = "Cluster Name" ... Value = Array of Cluster Attributes
$clusters = array();

# 2Key = "Cluster Name / Host Name" ... Value = Array of Host Attributes
$hosts = array();
# 2Key = "Cluster Name / Host Name" ... Value = Array of Host Attributes
$hosts_down = array();

# 3Key = "Cluster Name / Metric Name / Host Name" ... Value = Array of Metric Values
$metrics = array();

# 1Key = "Component" (gmetad | gmond) = Version string
$version = array();

# The web frontend version, from conf.php.
$version["webfrontend"] = "$majorversion.$minorversion.$microversion";

# A counter to help summarize a tree of grids.
$grid_depth = 0;
# The name of our local grid.
$self;
# A data structure to keep track of the Grid. A graph in adjacency-list
# format. Only grids can have children, clusters are leaf nodes.
$gridtree=array();
# A stack and pointer for constructing the gridtree graph.
$ancestors=array();
# Stands for 'parent' or 'predecessor'.
$p;

# True if the XML tree comes from a pre-grid gmetad. A new gmetad will
# wrap its output in a <GRID> tag.
$old = FALSE;
# True if we enter general mode (where we only have summary graphs of a grid).
# When this is FALSE, we operate like we did in previous versions.
$general_mode = FALSE;

function __start ($parser, $tagname, $attrs)
{
   global $clusters, $hosts, $metrics, $hosts_down, $version, $allgrids;
   global $grid_depth, $grids, $self, $old, $general_mode, $gridtree, $p, $ancestors;
   static $host, $grid, $cluster, $metric, $localtime, $TTL, $host_up;

   if ($tagname == "GANGLIA_XML")
      {
         $component = $attrs[SOURCE];
         $version[$component] = $attrs[VERSION];
      }
   else if ($tagname == "GRID")
      {
         $gridname = $attrs[NAME];
         $allgrids[$gridname] = $attrs;

         #echo "Added grid $grid, parent is $p<br>";
         #echo "Found a GRID: $attrs[NAME], grid_depth = $grid_depth<br>";
         #echo "auth=$attrs[AUTHORITY]<br>";

         if ($grid_depth==0) {
            $self = $gridname;
            $grids[$self] = $attrs;

            $p = $self;
            $gridtree[$self] = array();
         }
         else {
            # Maintain grid tree structure: a graph in adjacency-list format.
            array_push($ancestors, $p);
            $gridtree[$p][] = $gridname;
            $p = $gridname;
         }

         # At this stage, everything looks like it came from a new gmetad.
         if (!$general_mode and $grid_depth>0) $general_mode = TRUE;

         # Only record children grids, not deeper than that.
         if ($grid_depth==1) {
               $grid = $gridname;
               $grids[$grid] = $attrs;
         }

         $grid_depth++;
      }
   else if ($tagname == "CLUSTER")
      {
         $cluster = $attrs[NAME];
         #echo "Adding cluster $cluster, parent is $p<br>";

         # In new gmetads, GRID is the first tag after GANGLIA_XML.
         if (!$old and $grid_depth==0) {
             $old=TRUE;
             # Pretend we have a new style gmetad.
             $grid_depth++;
             $self=" ";
             $p = $self;
             $gridtree[$self] = array();
         }

         // Add cluster to grid tree graph.
         $gridtree[$p][] = $cluster;

         # Only keep track of immediate children clusters, not ones inside other grids.
         if (!$general_mode) {
            $clusters[$cluster] = $attrs;
         }

         # To determine health of older (pre 2.5.0) gmond hosts.
         $localtime = $attrs[LOCALTIME];
         $TTL = 15;
      }
   else if ($tagname == "HOST")
      {
         $host = $attrs[NAME];

         /* Check if host is UP or DOWN and place in appropriate array */
         $host_up = TRUE;
         $TN = $attrs[TN];
         $TMAX = $attrs[TMAX];
         # The new (>2.5.0) method to determine life, as used by gmetad.
         if ($TN and $TMAX) {
            if ($TN > $TMAX*4) 
              $host_up = FALSE;
         }
         # The old method.
         else {
            if (abs($localtime - $attrs[REPORTED]) > (4*$TTL))
              $host_up = FALSE;
         }
      
         # Record the number of nodes in $clusters and $grids, for efficiency.
         $hoststate = $host_up ? "HOSTS_UP" : "HOSTS_DOWN";
      
         if ($general_mode) {

             # Only keep summary state.
             $grids[$grid][$hoststate]++;

         }
         else {
      
            # We are in graphing mode. Keep detailed cluster state.
            $clusters[$cluster][$hoststate]++;
      
            if ($host_up) {
               $hosts[$cluster][$host] = $attrs;
            }
            else {
               $hosts_down[$cluster][$host] = $attrs;
            }
      
            # Put host attributes in the metrics array.
            $foo = array(
               "NAME"=>"ip",
               "VAL"=> $attrs[IP],
               "TYPE"=>"string"
               );
            $metrics[$cluster][$foo[NAME]][$host] = $foo;
            $foo = array(
               "NAME"=>"reported",
               "VAL"=> $attrs[REPORTED],
               "TYPE"=>"timestamp"
               );
            $metrics[$cluster][$foo[NAME]][$host] = $foo;
            $foo = array(
               "NAME"=>"gmond_started",
               "VAL"=> $attrs[GMOND_STARTED],
               "TYPE"=>"timestamp"
               );
            $metrics[$cluster][$foo[NAME]][$host] = $foo;
         }

         # Update top level summary
         $grids[$self][$hoststate]++;
      }

   else if ($tagname == "METRIC")
      { 
         $metric = $attrs[NAME]; 
         if ($host_up) {

             if ($general_mode) {
                  # Keep summaries for each child grid.
                  if (is_numeric($attrs[VAL])) {
                      $grids[$grid][$metric] += $attrs[VAL];
                  }
             }
             else {
                  # Keep full metric information in this case.
                  $metrics[$cluster][$metric][$host] = $attrs; 
             }

            # Update top level summary.
            if (is_numeric($attrs[VAL]))
               $grids[$self][$metric] += $attrs[VAL];
         }
      } 
} 

function __end ($parser, $tagname)
{
   global $grid_depth, $general_mode, $ancestors, $p;

   if ($tagname == "GRID")
      {
         $grid_depth--;

         $p = array_pop($ancestors);
         #echo "Ending grid, new parent is $p<br>";

         if ($grid_depth==1)
            $general_mode = FALSE;
      }
}

function Gmetad ()
{
   global $error, $parsetime;
   # From conf.php:
   global $ganglia_ip, $ganglia_port;

   # Parameters are optional
   # Defaults...
   $ip = $ganglia_ip;
   $port = $ganglia_port;
   $timeout = 3.0;
   $errstr = "";
   $errno  = "";

   switch( func_num_args() )
      {
         case 2:
            $port = func_get_arg(1);
         case 1:
            $ip   = func_get_arg(0);
      }

   $parser = xml_parser_create();
   xml_set_element_handler($parser, "__start", "__end" );

   $fp = fsockopen( $ip, $port, &$errno, &$errstr, $timeout);
   if (!$fp)
      {
         $error = "fsockopen error: $errstr";
         return FALSE;
      }
   else
      {
         $start = gettimeofday();
         while(!feof($fp))
            {
               $data = fgets($fp, 4096);
               if (!xml_parse($parser, $data, feof($fp)))
                  {
                     $error = sprintf("XML error: %s at %d",
                        xml_error_string(xml_get_error_code($parser)),
                        xml_get_current_line_number($parser));
                     fclose($fp);
                     return FALSE;
                  }
            }
         fclose($fp);
         $end = gettimeofday();
         $parsetime = ($end[sec] + $end[usec]/1e6) - ($start[sec] + $start[usec]/1e6);
      }

   return TRUE;
}

?>
