<?php
# $Id: show_node.php,v 1.2 2002/12/09 20:15:39 sacerdoti Exp $
#
# Shows information on a particular node, specified by
# node host name. Used with the physical_view page. 
# 
# Although this page repeats some information from
# host_view, it had the concept of Constant Metrics before
# the SLOPE=zero attribute.  It also uses style sheets for good
# looks. In the future, it may display process information for 
# the node as well.
#
# Copyright 2002 Federico Sacerdoti and the UC Regents.
#

# Host is specified in get_context.php.
$name=$host;

if (empty($name)) {
   print "<h1>Missing a Node Name</h1>";
   return;
}

$tpl = new TemplatePower( template("show_node.tpl") );
$tpl->prepare();
$tpl->assign("name",$name);

# Determine the liveness state of the node.
$state = $hosts[$cluster][$host] ? "UP" : "DOWN";

$class = ($state=="DOWN") ? "down" : "even";
$tpl->assign("class",$class);

# An array of [Metrics][Hostname][NAME|VAL|TYPE|UNITS|SOURCE].
$M=$metrics[$cluster];

$tpl->assign("ip", $M[ip][$name][VAL]);

# Find the host's physical location in the cluster.
$hostattrs = ($state=="UP") ? $hosts[$cluster][$name] : $hosts_down[$cluster][$name];
list($rack,$rank,$plane) = findlocation($hostattrs);
$location = ($rack<0) ? "Unknown" : "Rack $rack, Rank $rank, Plane $plane.";
$tpl->assign("location",$location);

# The metrics we need for this node.
$mem_totalMB = intval(intval($M[mem_total][$name][VAL])/1024);
$load_one=$M[load_one][$name][VAL];
$load_five=$M[load_five][$name][VAL];
$load_fifteen=$M[load_fifteen][$name][VAL];
$cpu_user=$M[cpu_user][$name][VAL];
$cpu_system=$M[cpu_system][$name][VAL];
$cpu_idle=$M[cpu_idle][$name][VAL];
$cpu_num=$M[cpu_num][$name][VAL];
# Cannot be zero, since we use it as a divisor.
if (!$cpu_num) { $cpu_num=1; }
$cpu_speed=$M[cpu_speed][$name][VAL];
$disk_total=$M[disk_total][$name][VAL];
$disk_free=$M[disk_free][$name][VAL];
$disk_use = $disk_total - $disk_free;
$disk_units=$M[disk_total][$name][UNITS];
$part_max_used=$M[part_max_used][$name][VAL];
# Disk metrics are newer (as of 2.5.0), so we check more carefully.
$disk = ($disk_total) ? "Using $disk_use of $disk_total $disk_units" : "Unknown";
$part_max = ($part_max_used) ? "$part_max_used% used." : "Unknown";
   

# Compute time of last heartbeat from node's dendrite.
$clustertime=$clusters[$cluster][LOCALTIME];
$heartbeat=$M[reported][$name][VAL];
$age = $clustertime - $heartbeat;
$s = ($age>1) ? "s" : "";
$tpl->assign("age","$age second$s");
if ($state=="DOWN") {
   $tpl->assign("down","<br><b>This node is down.</b>");
}

# The these hardware units should be more flexible.
$s = ($cpu_num>1) ? "s" : "";
$tpl->assign("s",$s);
$tpl->assign("cpu","$cpu_num x $cpu_speed Mhz");
$tpl->assign("mem","$mem_totalMB MB");
$tpl->assign("disk","$disk");
$tpl->assign("part_max_used", "$part_max");
$tpl->assign("load_one",$load_one);
$tpl->assign("load_five",$load_five);
$tpl->assign("load_fifteen",$load_fifteen);
$tpl->assign("cpu_user",$cpu_user);
$tpl->assign("cpu_system",$cpu_system);
$tpl->assign("cpu_idle",$cpu_idle);

# Choose a load color from a unix load value.
function loadindex($load) {
   global $cpu_num;
   # Highest color comes at a load of loadscalar*10.
   $loadscalar=0.2;
   $level=intval($load/($loadscalar*$cpu_num))+1;
   # Trim level to a max of 10.
   $level = $level > 10 ? "L10" : "L$level";
   return $level;
}
# Choose a load color from a 0-100 percentage.
function percentindex($val) {
   $level = intval($val/10 + 1);
   $level = $level>10 ? "L10" : "L$level";
   return $level;
}
$tpl->assign("load1",loadindex($load_one));
$tpl->assign("load5",loadindex($load_five));
$tpl->assign("load15",loadindex($load_fifteen));
$tpl->assign("user",percentindex($cpu_user));
$tpl->assign("sys",percentindex($cpu_system));
$tpl->assign("idle",percentindex(100 - $cpu_idle));

# Software metrics
$os_name=$M[os_name][$name][VAL];
$os_release=$M[os_release][$name][VAL];
$machine_type=$M[machine_type][$name][VAL];
$boottime=$M[boottime][$name][VAL];
$booted=date("F j, Y, g:i a", $boottime);
$uptime=uptime($name,$cluster);

# Turning into MBs. A MB is 1024 bytes.
$swap_free=$M[swap_free][$name][VAL]/1024.0;
$swap_total=sprintf("%.1f", $M[swap_total][$name][VAL]/1024.0);
$swap_used=sprintf("%.1f", $swap_total - $swap_free);

$tpl->assign("OS","$os_name $os_release ($machine_type)");
$tpl->assign("booted","$booted");
$tpl->assign("uptime", ($state=="UP") ? $uptime : "[down]");
$tpl->assign("swap","Using $swap_used of $swap_total MB swap.");

# For the back link.
$clusterURL=rawurlencode($cluster);
$tpl->assign("physical_view","./?p=$physical&c=$clusterURL");

# For the full host view link.
$tpl->assign("full_host_view","./?c=$clusterURL&h=$name&$get_metric_string");

# For the reload link.
$tpl->assign("self","$PHP_SELF?$QUERY_STRING");

$tpl->printToScreen();
?>
